/*
  mtetherd
  (c) 2010 Gregor Riepl <onitake@gmail.com>
  
  Tethering utility for Maemo
  
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <pwd.h>
#include <sys/types.h>
#include <gtk/gtk.h>
#include <hildon/hildon.h>

#include "status.h"

#define MTETHERD_STATUS_PLUGIN_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE(obj, TYPE_MTETHERD_STATUS_PLUGIN, MTetherDStatusPluginPrivate))

struct _MTetherDStatusPluginPrivate {
	GtkWidget *button;
	gboolean neton;
};

HD_DEFINE_PLUGIN_MODULE(MTetherDStatusPlugin, mtetherd_status_plugin, HD_TYPE_STATUS_MENU_ITEM);

static void mtetherd_status_plugin_class_finalize(MTetherDStatusPluginClass *klass) { }

static void mtetherd_status_plugin_class_init(MTetherDStatusPluginClass *klass) {
	g_type_class_add_private(klass, sizeof(MTetherDStatusPluginPrivate));
}

static gboolean get_usbnet_enabled(MTetherDStatusPlugin *plugin) {
	FILE *fp = fopen("/proc/modules", "r");
	if (!fp) {
		// fallback (useless?)
		return plugin->priv->neton;
	}
	gboolean found = FALSE;
	char *line = NULL;
	while (!found && getline(&line, NULL, fp) != -1) {
		if (strncmp(line, "g_ether", 7) == 0) {
			found = TRUE;
		}
		free(line);
		line = NULL;
	}
	fclose(fp);
	return found;
}

static void set_button_text(GtkWidget *button, gboolean enabled) {
	if (enabled) {
		gtk_button_set_label(GTK_BUTTON(button), "Disable USB Networking");
	} else {
		gtk_button_set_label(GTK_BUTTON(button), "Enable USB Networking");
	}
}

static gboolean launch_usbnet_script(gboolean enable) {
	const char *arg;
	if (enable) {
		arg = "/usr/sbin/mtetherd-usbnet-enable.sh";
	} else {
		arg = "/usr/sbin/mtetherd-usbnet-disable.sh";
	}
	const char *command[] = { "/usr/bin/sudo", arg, NULL };
	pid_t pid = fork();
	if (pid == 0) {
		if (execv(command[0], (char **const) command) == -1) {
			exit(1);
		}
	} else if (pid == -1) {
		// error forking
		return FALSE;
	}
	// launch ok
	return TRUE;
}

static void enable_usb_net_clicked(GtkWidget *button, gpointer user) {
	MTetherDStatusPlugin *plugin = MTETHERD_STATUS_PLUGIN(user);

	if (button == plugin->priv->button) {
		gboolean enabled = get_usbnet_enabled(plugin);
		set_button_text(plugin->priv->button, enabled);
		if (enabled) {
			if (launch_usbnet_script(FALSE)) {
				// launch ok
				plugin->priv->neton = FALSE;
			}
		} else {
			if (launch_usbnet_script(TRUE)) {
				plugin->priv->neton = TRUE;
			}
		}
	}
}

static void mtetherd_status_plugin_init(MTetherDStatusPlugin *plugin) {
	plugin->priv = MTETHERD_STATUS_PLUGIN_GET_PRIVATE(plugin);

	plugin->priv->neton = FALSE;
	plugin->priv->button = gtk_button_new();
	set_button_text(plugin->priv->button, plugin->priv->neton);
	g_signal_connect(plugin->priv->button, "clicked", G_CALLBACK(enable_usb_net_clicked), plugin);
	gtk_container_add(GTK_CONTAINER(plugin), plugin->priv->button);
	gtk_widget_show_all(plugin->priv->button);
	gtk_widget_show(GTK_WIDGET(plugin));
}

