#!/bin/sh
#
# /usr/bin/tethering.sh
#
# Enable tethering on USB network and Bluetooth PAN.
#
# Note that the INETDEV must be up and running when this script is
# started, or forwarding will not be enabled.
# Having tethering on multiple devices concurrently is possible,
# but forwarding will be disabled once the first device disconnetcs.
# A better check is necessary.
#
# Put this into /etc/dbus-scripts/tethering :
# /usr/bin/tethering.sh * * org.freedesktop.Hal.Manager DeviceAdded
# /usr/bin/tethering.sh * * org.freedesktop.Hal.Manager DeviceRemoved
#
# dbus-scripts doesn't give us the event path, so we have to use a more
# convoluted method of getting the network interface.
#

EVENT="$4"
UDI="$5"
HALDEV="$(echo $UDI | sed 's#.*/\([0-9a-zA-Z_]*\)#\1#')"
LOG="/tmp/tethering.log"
INETDEV="gprs0"
RUNFILE="/var/run/tethering.$HALDEV.pid"
IFACEFILE="/var/run/tethering.$HALDEV.iface"

if [ "$EVENT" = "DeviceAdded" ]; then
	IFACE=$(hal-get-property --udi $UDI --key net.interface)
	case $IFACE in
		bnep0 )
			IF_ADDRESS=192.168.254.254
			IF_RANGE=192.168.254.1,192.168.254.254
		;;
	 	usb0 )
			IF_ADDRESS=192.168.253.254
			IF_RANGE=192.168.253.1,192.168.253.254
		;;
		* )
			exit 0
		;;
	esac

	echo "$(date) Enabling tethering on interface $IFACE" >> $LOG
	echo "$IFACE" > $IFACEFILE
	ifconfig "$IFACE" "$IF_ADDRESS"
	/sbin/modprobe ipt_MASQUERADE
	/usr/sbin/iptables -t nat -A POSTROUTING -o $INETDEV -j MASQUERADE
	/usr/sbin/start-stop-daemon -S -p "$RUNFILE" -b -x /usr/sbin/dnsmasq -- -x "$RUNFILE" -k -I lo -i "$IFACE" -a $IF_ADDRESS -z -F $IF_RANGE,3600
	echo 1 > /proc/sys/net/ipv4/conf/$IFACE/forwarding
	echo 1 > /proc/sys/net/ipv4/conf/$INETDEV/forwarding
fi

if [ "$EVENT" = "DeviceRemoved" ]; then
	if [ ! -f "$IFACEFILE" ]; then
		exit 0
	fi
	IFACE="$(cat $IFACEFILE)"
	rm "$IFACEFILE"
	echo "$(date) Disabling tethering on device $IFACE" >> $LOG
	echo 0 > /proc/sys/net/ipv4/conf/$INETDEV/forwarding
	/usr/sbin/start-stop-daemon -K -p "$RUNFILE" -x /usr/sbin/dnsmasq
	/usr/sbin/iptables -t nat -D POSTROUTING -o $INETDEV -j MASQUERADE
fi

exit 0
